import datetime
import uuid
from collections import OrderedDict
from decimal import Decimal
from enum import Enum
from json import JSONDecoder, JSONEncoder
from typing import (
    Any,
    Callable,
    Dict,
    Generator,
    Generic,
    Iterable,
    List,
    Mapping,
    MutableMapping,
    NoReturn,
    Optional,
    Pattern,
    Protocol,
    Sequence,
    Set,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from django.core.files.base import File
from django.db import models
from django.forms import ImageField as DjangoImageField  # noqa: F401
from typing_extensions import Final

from rest_framework.serializers import BaseSerializer
from rest_framework.validators import Validator

class _Empty(Enum):
    sentinel = 0

empty: Final = _Empty.sentinel

class BuiltinSignatureError(Exception): ...

class CreateOnlyDefault:
    requires_context: bool = ...
    default: Any = ...
    def __init__(self, default: Any) -> None: ...
    def __call__(self, serializer_field: Field): ...

class CurrentUserDefault:
    requires_context: bool = ...
    def __call__(self, serializer_field: Field): ...

class SkipField(Exception): ...

class Option(Protocol):
    start_option_group: bool = ...
    end_option_group: bool = ...
    label: str
    value: str
    display_text: str

def is_simple_callable(obj: Callable) -> bool: ...
def get_attribute(instance: Any, attrs: Optional[List[str]]) -> Any: ...
def set_value(dictionary: MutableMapping[str, Any], keys: Sequence[str], value: Any) -> None: ...
def to_choices_dict(choices: Iterable[Any]) -> OrderedDict: ...
def flatten_choices_dict(choices: Dict[Any, Any]) -> OrderedDict: ...
def iter_options(
    grouped_choices: OrderedDict, cutoff: Optional[int] = ..., cutoff_text: Optional[str] = ...
) -> Generator[Option, None, None]: ...
def get_error_detail(exc_info: Any) -> Any: ...

REGEX_TYPE: Pattern
NOT_READ_ONLY_WRITE_ONLY: str
NOT_READ_ONLY_REQUIRED: str
NOT_REQUIRED_DEFAULT: str
USE_READONLYFIELD: str
MISSING_ERROR_MESSAGE: str

_IN = TypeVar("_IN")  # Instance Type
_VT = TypeVar("_VT")  # Value Type
_DT = TypeVar("_DT")  # Data Type
_RP = TypeVar("_RP")  # Representation Type

class SupportsToPython(Protocol):
    def to_python(self, value: Any) -> Any: ...

_DefaultInitial = Union[_VT, Callable[[], _VT], None, _Empty]

class Field(Generic[_VT, _DT, _RP, _IN]):
    allow_null: bool = ...
    default: Optional[_VT] = ...
    default_empty_html: Any = ...
    default_error_messages: Dict[str, str] = ...
    default_validators: List[Validator[_VT]] = ...
    error_messages: Dict[str, str] = ...
    field_name: Optional[str] = ...
    help_text: Optional[str] = ...
    initial: Optional[Union[_VT, Callable[[], _VT]]] = ...
    label: Optional[str]
    parent: BaseSerializer
    read_only: bool
    required: bool
    source: Optional[Union[Callable, str]]
    source_attrs: List[str] = ...
    style: Dict[str, Any]
    write_only: bool
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[_VT] = ...,
        initial: _DefaultInitial[_VT] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[_VT]]] = ...,
        allow_null: bool = ...,
    ): ...
    def bind(self, field_name: str, parent: BaseSerializer) -> None: ...
    @property
    def validators(self) -> List[Validator[_VT]]: ...
    @validators.setter
    def validators(self, validators: List[Validator[_VT]]) -> None: ...
    def get_validators(self) -> List[Validator[_VT]]: ...
    def get_initial(self) -> Optional[_VT]: ...
    def get_value(self, dictionary: Mapping[Any, Any]) -> Any: ...
    def get_attribute(self, instance: _IN) -> Optional[_RP]: ...
    def get_default(self) -> Optional[_VT]: ...
    def validate_empty_values(self, data: Any) -> Tuple[bool, Any]: ...
    def run_validation(self, data: Any = ...) -> Any: ...
    def run_validators(self, value: Any) -> None: ...
    def to_internal_value(self, data: _DT) -> _VT: ...
    def to_representation(self, value: _VT) -> _RP: ...
    def fail(self, key: str, **kwargs: Any) -> NoReturn: ...
    @property
    def root(self) -> BaseSerializer: ...
    @property
    def context(self) -> Dict[str, Any]: ...
    def __new__(cls, *args: Any, **kwargs: Any) -> Field: ...
    def __deepcopy__(self, memo: Mapping[Any, Any]) -> Field: ...

class BooleanField(
    Field[
        bool,
        Union[str, bool, int],
        bool,
        Any,
    ]
):
    TRUE_VALUES: Set[Union[str, bool, int]] = ...
    FALSE_VALUES: Set[Union[str, bool, int, float]] = ...
    NULL_VALUES: Set[Union[str, None]] = ...

class NullBooleanField(
    Field[
        Union[bool, None],
        Optional[Union[str, bool, int]],
        bool,
        Any,
    ]
):
    TRUE_VALUES: Set[Union[str, bool, int]] = ...
    FALSE_VALUES: Set[Union[str, bool, int, float]] = ...
    NULL_VALUES: Set[Union[str, None]] = ...

class CharField(Field[str, str, str, Any]):
    allow_blank: bool = ...
    trim_whitespace: bool = ...
    max_length: Optional[int] = ...
    min_length: Optional[int] = ...
    def __init__(
        self,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[str] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[str]]] = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        max_length: int = ...,
        min_length: Optional[int] = ...,
    ): ...

class EmailField(CharField): ...

class RegexField(CharField):
    def __init__(
        self,
        regex: Union[str, Pattern],
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[str] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[str]]] = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        max_length: int = ...,
        min_length: Optional[int] = ...,
    ): ...

class SlugField(CharField):
    allow_unicode: bool = ...
    def __init__(
        self,
        allow_unicode: bool = ...,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[str] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[str]]] = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        max_length: int = ...,
        min_length: Optional[int] = ...,
    ): ...

class URLField(CharField): ...

class UUIDField(Field[uuid.UUID, Union[uuid.UUID, str, int], str, Any]):
    valid_formats: Sequence[str] = ...
    uuid_format: str
    def __init__(
        self,
        *,
        format: Optional[str] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[uuid.UUID] = ...,
        initial: _DefaultInitial[uuid.UUID] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[uuid.UUID]]] = ...,
        allow_null: bool = ...,
    ): ...

class IPAddressField(CharField):
    protocol: str
    unpack_ipv4: bool
    def __init__(
        self,
        protocol: str = ...,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[str] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[str]]] = ...,
        allow_null: bool = ...,
        allow_blank: bool = ...,
        trim_whitespace: bool = ...,
        max_length: int = ...,
        min_length: Optional[int] = ...,
    ): ...

class IntegerField(Field[int, Union[float, int, str], int, Any]):
    MAX_STRING_LENGTH: int = ...
    re_decimal: Pattern = ...
    max_value: Optional[int] = ...
    min_value: Optional[int] = ...
    def __init__(
        self,
        *,
        max_value: int = ...,
        min_value: int = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[int] = ...,
        initial: _DefaultInitial[int] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[int]]] = ...,
        allow_null: bool = ...,
    ): ...

class FloatField(Field[float, Union[float, int, str], str, Any]):
    MAX_STRING_LENGTH: int = ...
    re_decimal: Pattern = ...
    max_value: Optional[float] = ...
    min_value: Optional[float] = ...
    def __init__(
        self,
        *,
        max_value: float = ...,
        min_value: float = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[float] = ...,
        initial: _DefaultInitial[float] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[float]]] = ...,
        allow_null: bool = ...,
    ): ...

class DecimalField(Field[Decimal, Union[int, float, str, Decimal], str, Any]):
    MAX_STRING_LENGTH: int = ...
    max_digits: Optional[int]
    decimal_places: Optional[int]
    coerce_to_string: Optional[bool]
    max_value: Optional[Union[Decimal, int, float]]
    min_value: Optional[Union[Decimal, int, float]]
    localize: bool
    rounding: Optional[str]
    max_whole_digits = Optional[int]
    def __init__(
        self,
        max_digits: Optional[int],
        decimal_places: Optional[int],
        coerce_to_string: bool = ...,
        max_value: Union[Decimal, int, float] = ...,
        min_value: Union[Decimal, int, float] = ...,
        localize: bool = ...,
        rounding: Optional[str] = ...,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[Decimal] = ...,
        initial: _DefaultInitial[Decimal] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[Decimal]]] = ...,
        allow_null: bool = ...,
    ): ...
    def validate_precision(self, value: Decimal) -> Decimal: ...
    def quantize(self, value: Decimal) -> Decimal: ...

class DateTimeField(Field[datetime.datetime, Union[datetime.datetime, str], str, Any]):
    datetime_parser: Callable[[str, str], datetime.datetime] = ...
    format: Optional[str] = ...
    input_formats: Sequence[str] = ...
    timezone: datetime.tzinfo = ...
    def __init__(
        self,
        format: Optional[str] = ...,
        input_formats: Sequence[str] = ...,
        default_timezone: Optional[datetime.tzinfo] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[datetime.datetime] = ...,
        initial: _DefaultInitial[datetime.datetime] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[datetime.datetime]]] = ...,
        allow_null: bool = ...,
    ): ...
    def enforce_timezone(self, value: datetime.datetime) -> datetime.datetime: ...
    def default_timezone(self) -> Optional[str]: ...

class DateField(Field[datetime.date, Union[datetime.date, str], str, Any]):
    datetime_parser: Callable[[str, str], datetime.datetime] = ...
    format: Optional[str] = ...
    input_formats: Sequence[str] = ...
    def __init__(
        self,
        format: Optional[str] = ...,
        input_formats: Sequence[str] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[datetime.date] = ...,
        initial: _DefaultInitial[datetime.date] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[datetime.date]]] = ...,
        allow_null: bool = ...,
    ): ...

class TimeField(Field[datetime.time, Union[datetime.time, str], str, Any]):
    datetime_parser: Callable[[str, str], datetime.datetime] = ...
    format: Optional[str] = ...
    input_formats: Sequence[str] = ...
    def __init__(
        self,
        format: Optional[str] = ...,
        input_formats: Sequence[str] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[datetime.time] = ...,
        initial: _DefaultInitial[datetime.time] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[datetime.time]]] = ...,
        allow_null: bool = ...,
    ): ...

class DurationField(Field[datetime.timedelta, Union[datetime.timedelta, str], str, Any]):
    max_value: Optional[datetime.timedelta] = ...
    min_value: Optional[datetime.timedelta] = ...
    def __init__(
        self,
        *,
        max_value: Union[datetime.timedelta, int, float] = ...,
        min_value: Union[datetime.timedelta, int, float] = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[datetime.timedelta] = ...,
        initial: _DefaultInitial[datetime.timedelta] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[datetime.timedelta]]] = ...,
        allow_null: bool = ...,
    ): ...

class ChoiceField(Field[str, Union[str, int, Tuple[Union[str, int], Union[str, int, tuple]]], str, Any]):
    html_cutoff: Optional[int] = ...
    html_cutoff_text: Optional[str] = ...
    allow_blank: bool = ...
    grouped_choices: OrderedDict = ...
    choice_strings_to_values: Dict[str, Any] = ...
    _choices: OrderedDict = ...
    def __init__(
        self,
        choices: Iterable[Any],
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[Union[str, int]] = ...,
        initial: _DefaultInitial[Union[str, int]] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[Any]]] = ...,
        allow_null: bool = ...,
        html_cutoff: int = ...,
        html_cutoff_text: str = ...,
        allow_blank: bool = ...,
    ): ...
    def iter_options(self) -> Iterable[Option]: ...
    def _get_choices(self) -> Dict[Any, Any]: ...
    def _set_choices(self, choices: Iterable[Any]) -> None: ...
    choices = property(_get_choices, _set_choices)

class MultipleChoiceField(
    ChoiceField,
    Field[
        str,
        Sequence[Union[str, int, Tuple[Union[str, int], Union[str, int]]]],
        Sequence[Union[str, Tuple[Union[str, int], Union[str, int]]]],
        Any,
    ],
):
    allow_empty: bool = ...
    def __init__(
        self,
        choices: Iterable[Any],
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[Union[Set[Union[str, int]], Set[str], Set[int]]] = ...,
        initial: _DefaultInitial[Union[Set[Union[str, int]], Set[str], Set[int]]] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[Any]]] = ...,
        allow_null: bool = ...,
        html_cutoff: int = ...,
        html_cutoff_text: str = ...,
        allow_blank: bool = ...,
        allow_empty: bool = ...,
    ): ...

class FilePathField(ChoiceField):
    def __init__(
        self,
        path: str,
        match: str = ...,
        recursive: bool = ...,
        allow_files: bool = ...,
        allow_folders: bool = ...,
        required: bool = ...,
        read_only: bool = ...,
        write_only: bool = ...,
        default: _DefaultInitial[str] = ...,
        initial: _DefaultInitial[str] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[Any]]] = ...,
        allow_null: bool = ...,
        html_cutoff: int = ...,
        html_cutoff_text: str = ...,
        allow_blank: bool = ...,
    ): ...

class FileField(Field[File, File, Union[str, None], Any]):  # this field can return None without raising!
    max_length: int = ...
    allow_empty_file: bool = ...
    use_url: bool = ...
    def __init__(
        self,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[File] = ...,
        initial: _DefaultInitial[File] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[File]]] = ...,
        allow_null: bool = ...,
        max_length: int = ...,
        allow_empty_file: bool = ...,
        use_url: bool = ...,
    ): ...

class ImageField(FileField):
    _DjangoImageField: SupportsToPython = ...
    def __init__(
        self,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[File] = ...,
        initial: _DefaultInitial[File] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[File]]] = ...,
        allow_null: bool = ...,
        max_length: int = ...,
        allow_empty_file: bool = ...,
        use_url: bool = ...,
        _DjangoImageField: Type[SupportsToPython] = ...,
    ): ...

class _UnvalidatedField(Field): ...

class ListField(Field[List[Any], List[Any], List[Any], Any]):
    child: Field = ...
    allow_empty: bool = ...
    max_length: Optional[int] = ...
    min_length: Optional[int] = ...
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[List[Any]] = ...,
        initial: _DefaultInitial[List[Any]] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[List[Any]]]] = ...,
        allow_null: bool = ...,
        *,
        child: Field = ...,
        allow_empty: bool = ...,
        max_length: int = ...,
        min_length: int = ...,
    ): ...
    def run_child_validation(self, data: List[Mapping[Any, Any]]) -> Any: ...

class DictField(Field[Dict[Any, Any], Dict[Any, Any], Dict[Any, Any], Any]):
    child: Field = ...
    allow_empty: bool = ...
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[Dict[Any, Any]] = ...,
        initial: _DefaultInitial[Dict[Any, Any]] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[Dict[Any, Any]]]] = ...,
        allow_null: bool = ...,
        *,
        child: Field = ...,
        allow_empty: bool = ...,
    ): ...
    def run_child_validation(self, data: Any) -> Any: ...

class HStoreField(DictField):
    child: CharField = ...

class JSONField(
    Field[Union[Dict[str, Any], List[Dict[str, Any]]], Union[Dict[str, Any], List[Dict[str, Any]]], str, Any]
):
    binary: bool = ...
    encoder: Optional[JSONEncoder] = ...
    decoder: Optional[JSONDecoder] = ...
    def __init__(
        self,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: _DefaultInitial[Mapping[Any, Any]] = ...,
        initial: _DefaultInitial[Mapping[Any, Any]] = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[Any]]] = ...,
        allow_null: bool = ...,
        *,
        binary: bool = ...,
        encoder: Optional[JSONEncoder] = ...,
        decoder: Optional[JSONDecoder] = ...,
    ): ...

class ReadOnlyField(Field): ...
class HiddenField(Field): ...

class SerializerMethodField(Field):
    method_name: str = ...
    def __init__(
        self,
        method_name: Optional[str] = ...,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[Any]]] = ...,
        allow_null: bool = ...,
    ): ...

class ModelField(Field):
    model_field: models.Field = ...
    max_length: int = ...
    def __init__(
        self,
        model_field: models.Field,
        *,
        read_only: bool = ...,
        write_only: bool = ...,
        required: bool = ...,
        default: Any = ...,
        initial: Any = ...,
        source: str = ...,
        label: str = ...,
        help_text: str = ...,
        style: Dict[str, Any] = ...,
        error_messages: Dict[str, str] = ...,
        validators: Optional[Sequence[Validator[Any]]] = ...,
        allow_null: bool = ...,
        max_length: int = ...,
    ): ...



from typing import Any, Dict, List, Tuple, Union

from .base import BaseCommand as BaseCommand
from .base import CommandError as CommandError

def find_commands(management_dir: str) -> List[str]: ...
def load_command_class(app_name: str, name: str) -> BaseCommand: ...
def get_commands() -> Dict[str, Union[str, BaseCommand]]: ...
def call_command(command_name: Union[Tuple[str], BaseCommand, str], *args: Any, **options: Any) -> str: ...

class ManagementUtility:
    argv: List[str] = ...
    prog_name: str = ...
    settings_exception: None = ...
    def __init__(self, argv: List[str] = ...) -> None: ...
    def main_help_text(self, commands_only: bool = ...): ...
    def fetch_command(self, subcommand: str) -> BaseCommand: ...
    def autocomplete(self) -> None: ...
    def execute(self) -> None: ...

def execute_from_command_line(argv: List[str] = ...) -> None: ...



from typing import Any

from django.contrib.gis.geos.geometry import GEOSGeometry as GEOSGeometry

class Polygon(GEOSGeometry):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self): ...
    @classmethod
    def from_bbox(cls, bbox: Any): ...
    @property
    def num_interior_rings(self): ...
    exterior_ring: Any = ...
    shell: Any = ...
    @property
    def tuple(self): ...
    coords: Any = ...
    @property
    def kml(self): ...



from typing import Any, Optional

from django.contrib.gis.geometry import hex_regex as hex_regex  # noqa: F401
from django.contrib.gis.geometry import json_regex as json_regex
from django.contrib.gis.geometry import wkt_regex as wkt_regex
from django.contrib.gis.geos.base import GEOSBase as GEOSBase
from django.contrib.gis.geos.mutable_list import ListMixin as ListMixin

class GEOSGeometryBase(GEOSBase):
    ptr_type: Any = ...
    destructor: Any = ...
    has_cs: bool = ...
    __class__: Any = ...
    def __init__(self, ptr: Any, cls: Any) -> None: ...
    def __copy__(self): ...
    def __deepcopy__(self, memodict: Any): ...
    @staticmethod
    def from_ewkt(ewkt: Any): ...
    @classmethod
    def from_gml(cls, gml_string: Any): ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> Any: ...
    def __or__(self, other: Any): ...
    def __and__(self, other: Any): ...
    def __sub__(self, other: Any): ...
    def __xor__(self, other: Any): ...
    @property
    def coord_seq(self): ...
    @property
    def geom_type(self): ...
    @property
    def geom_typeid(self): ...
    @property
    def num_geom(self): ...
    @property
    def num_coords(self): ...
    @property
    def num_points(self): ...
    @property
    def dims(self): ...
    def normalize(self) -> None: ...
    @property
    def empty(self): ...
    @property
    def hasz(self): ...
    @property
    def ring(self): ...
    @property
    def simple(self): ...
    @property
    def valid(self): ...
    @property
    def valid_reason(self): ...
    def contains(self, other: Any): ...
    def covers(self, other: Any): ...
    def crosses(self, other: Any): ...
    def disjoint(self, other: Any): ...
    def equals(self, other: Any): ...
    def equals_exact(self, other: Any, tolerance: float = ...): ...
    def intersects(self, other: Any): ...
    def overlaps(self, other: Any): ...
    def relate_pattern(self, other: Any, pattern: Any): ...
    def touches(self, other: Any): ...
    def within(self, other: Any): ...
    @property
    def srid(self): ...
    @srid.setter
    def srid(self, srid: Any) -> None: ...
    @property
    def ewkt(self): ...
    @property
    def wkt(self): ...
    @property
    def hex(self): ...
    @property
    def hexewkb(self): ...
    @property
    def json(self): ...
    geojson: Any = ...
    @property
    def wkb(self): ...
    @property
    def ewkb(self): ...
    @property
    def kml(self): ...
    @property
    def prepared(self): ...
    @property
    def ogr(self): ...
    @property
    def srs(self): ...
    @property
    def crs(self): ...
    ptr: Any = ...
    def transform(self, ct: Any, clone: bool = ...): ...
    @property
    def boundary(self): ...
    def buffer(self, width: Any, quadsegs: int = ...): ...
    def buffer_with_style(
        self, width: Any, quadsegs: int = ..., end_cap_style: int = ..., join_style: int = ..., mitre_limit: float = ...
    ): ...
    @property
    def centroid(self): ...
    @property
    def convex_hull(self): ...
    def difference(self, other: Any): ...
    @property
    def envelope(self): ...
    def intersection(self, other: Any): ...
    @property
    def point_on_surface(self): ...
    def relate(self, other: Any): ...
    def simplify(self, tolerance: float = ..., preserve_topology: bool = ...): ...
    def sym_difference(self, other: Any): ...
    @property
    def unary_union(self): ...
    def union(self, other: Any): ...
    @property
    def area(self): ...
    def distance(self, other: Any): ...
    @property
    def extent(self): ...
    @property
    def length(self): ...
    def clone(self): ...

class LinearGeometryMixin:
    def interpolate(self, distance: Any): ...
    def interpolate_normalized(self, distance: Any): ...
    def project(self, point: Any): ...
    def project_normalized(self, point: Any): ...
    @property
    def merged(self): ...
    @property
    def closed(self): ...

class GEOSGeometry(GEOSGeometryBase, ListMixin):
    srid: Any = ...
    def __init__(self, geo_input: Any, srid: Optional[Any] = ...) -> None: ...



import sys
from _typeshed import StrOrBytesPath
from cProfile import Profile as _cProfile
from profile import Profile
from typing import IO, Any, Iterable, TypeVar, Union, overload

_Selector = Union[str, float, int]
_T = TypeVar("_T", bound=Stats)

if sys.version_info >= (3, 7):
    from enum import Enum
    class SortKey(str, Enum):
        CALLS: str
        CUMULATIVE: str
        FILENAME: str
        LINE: str
        NAME: str
        NFL: str
        PCALLS: str
        STDNAME: str
        TIME: str

class Stats:
    sort_arg_dict_default: dict[str, tuple[Any, str]]
    def __init__(
        self: _T,
        __arg: None | str | Profile | _cProfile = ...,
        *args: None | str | Profile | _cProfile | _T,
        stream: IO[Any] | None = ...,
    ) -> None: ...
    def init(self, arg: None | str | Profile | _cProfile) -> None: ...
    def load_stats(self, arg: None | str | Profile | _cProfile) -> None: ...
    def get_top_level_stats(self) -> None: ...
    def add(self: _T, *arg_list: None | str | Profile | _cProfile | _T) -> _T: ...
    def dump_stats(self, filename: StrOrBytesPath) -> None: ...
    def get_sort_arg_defs(self) -> dict[str, tuple[tuple[tuple[int, int], ...], str]]: ...
    @overload
    def sort_stats(self: _T, field: int) -> _T: ...
    @overload
    def sort_stats(self: _T, *field: str) -> _T: ...
    def reverse_order(self: _T) -> _T: ...
    def strip_dirs(self: _T) -> _T: ...
    def calc_callees(self) -> None: ...
    def eval_print_amount(self, sel: _Selector, list: list[str], msg: str) -> tuple[list[str], str]: ...
    def get_print_list(self, sel_list: Iterable[_Selector]) -> tuple[int, list[str]]: ...
    def print_stats(self: _T, *amount: _Selector) -> _T: ...
    def print_callees(self: _T, *amount: _Selector) -> _T: ...
    def print_callers(self: _T, *amount: _Selector) -> _T: ...
    def print_call_heading(self, name_size: int, column_title: str) -> None: ...
    def print_call_line(self, name_size: int, source: str, call_dict: dict[str, Any], arrow: str = ...) -> None: ...
    def print_title(self) -> None: ...
    def print_line(self, func: str) -> None: ...



from typing import Any, Dict

from django.apps import apps as apps
from django.conf import settings as settings
from django.core.management.base import BaseCommand as BaseCommand
from django.core.management.base import CommandError as CommandError
from django.core.management.base import no_translations as no_translations
from django.db import DEFAULT_DB_ALIAS as DEFAULT_DB_ALIAS
from django.db import OperationalError as OperationalError
from django.db import connections as connections
from django.db import router as router
from django.db.migrations import Migration as Migration
from django.db.migrations.autodetector import MigrationAutodetector as MigrationAutodetector
from django.db.migrations.loader import MigrationLoader as MigrationLoader
from django.db.migrations.questioner import InteractiveMigrationQuestioner as InteractiveMigrationQuestioner
from django.db.migrations.questioner import MigrationQuestioner as MigrationQuestioner
from django.db.migrations.questioner import NonInteractiveMigrationQuestioner as NonInteractiveMigrationQuestioner
from django.db.migrations.state import ProjectState as ProjectState
from django.db.migrations.utils import get_migration_name_timestamp as get_migration_name_timestamp
from django.db.migrations.writer import MigrationWriter as MigrationWriter

class Command(BaseCommand):
    verbosity: int = ...
    interactive: bool = ...
    dry_run: bool = ...
    merge: bool = ...
    empty: bool = ...
    migration_name: str = ...
    include_header: bool = ...
    def write_migration_files(self, changes: Dict[str, Any]) -> None: ...
    def handle_merge(self, loader: MigrationLoader, conflicts: Dict[str, Any]) -> None: ...



from typing import Any, Optional

from django.db.models.query_utils import DeferredAttribute

class SpatialProxy(DeferredAttribute):
    def __init__(self, klass: Any, field: Any, load_func: Optional[Any] = ...) -> None: ...
    def __get__(self, instance: Any, cls: Optional[Any] = ...): ...
    def __set__(self, instance: Any, value: Any): ...



import sys
from typing import NamedTuple, Tuple

if sys.platform != "win32":
    class error(Exception): ...
    RLIM_INFINITY: int
    def getrlimit(resource: int) -> Tuple[int, int]: ...
    def setrlimit(resource: int, limits: Tuple[int, int]) -> None: ...
    RLIMIT_CORE: int
    RLIMIT_CPU: int
    RLIMIT_FSIZE: int
    RLIMIT_DATA: int
    RLIMIT_STACK: int
    RLIMIT_RSS: int
    RLIMIT_NPROC: int
    RLIMIT_NOFILE: int
    RLIMIT_OFILE: int
    RLIMIT_MEMLOCK: int
    RLIMIT_VMEM: int
    RLIMIT_AS: int
    class _RUsage(NamedTuple):
        ru_utime: float
        ru_stime: float
        ru_maxrss: int
        ru_ixrss: int
        ru_idrss: int
        ru_isrss: int
        ru_minflt: int
        ru_majflt: int
        ru_nswap: int
        ru_inblock: int
        ru_oublock: int
        ru_msgsnd: int
        ru_msgrcv: int
        ru_nsignals: int
        ru_nvcsw: int
        ru_nivcsw: int
    def getrusage(who: int) -> _RUsage: ...
    def getpagesize() -> int: ...
    RUSAGE_SELF: int
    RUSAGE_CHILDREN: int
    RUSAGE_BOTH: int



import sys
from _typeshed import Self, StrOrBytesPath
from types import TracebackType
from typing import (
    IO,
    Any,
    AsyncGenerator,
    AsyncIterator,
    Awaitable,
    Callable,
    ContextManager,
    Generator,
    Generic,
    Iterator,
    Optional,
    Protocol,
    Type,
    TypeVar,
    overload,
)
from typing_extensions import ParamSpec

AbstractContextManager = ContextManager
if sys.version_info >= (3, 7):
    from typing import AsyncContextManager

    AbstractAsyncContextManager = AsyncContextManager

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)
_T_io = TypeVar("_T_io", bound=Optional[IO[str]])
_F = TypeVar("_F", bound=Callable[..., Any])
_P = ParamSpec("_P")

_ExitFunc = Callable[[Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]], bool]
_CM_EF = TypeVar("_CM_EF", AbstractContextManager[Any], _ExitFunc)

class ContextDecorator:
    def __call__(self, func: _F) -> _F: ...

class _GeneratorContextManager(AbstractContextManager[_T_co], ContextDecorator, Generic[_T_co]):
    # In Python <= 3.6, __init__ and all instance attributes are defined directly on this class.
    # In Python >= 3.7, __init__ and all instance attributes are inherited from _GeneratorContextManagerBase
    # _GeneratorContextManagerBase is more trouble than it's worth to include in the stub; see #6676
    def __init__(self, func: Callable[..., Iterator[_T_co]], args: tuple[Any, ...], kwds: dict[str, Any]) -> None: ...
    gen: Generator[_T_co, Any, Any]
    func: Callable[..., Generator[_T_co, Any, Any]]
    args: tuple[Any, ...]
    kwds: dict[str, Any]

def contextmanager(func: Callable[_P, Iterator[_T_co]]) -> Callable[_P, _GeneratorContextManager[_T_co]]: ...

if sys.version_info >= (3, 10):
    _AF = TypeVar("_AF", bound=Callable[..., Awaitable[Any]])
    class AsyncContextDecorator:
        def __call__(self, func: _AF) -> _AF: ...
    class _AsyncGeneratorContextManager(AbstractAsyncContextManager[_T_co], AsyncContextDecorator, Generic[_T_co]):
        # __init__ and these attributes are actually defined in the base class _GeneratorContextManagerBase,
        # which is more trouble than it's worth to include in the stub (see #6676)
        def __init__(self, func: Callable[..., AsyncIterator[_T_co]], args: tuple[Any, ...], kwds: dict[str, Any]) -> None: ...
        gen: AsyncGenerator[_T_co, Any]
        func: Callable[..., AsyncGenerator[_T_co, Any]]
        args: tuple[Any, ...]
        kwds: dict[str, Any]

elif sys.version_info >= (3, 7):
    class _AsyncGeneratorContextManager(AbstractAsyncContextManager[_T_co], Generic[_T_co]):
        def __init__(self, func: Callable[..., AsyncIterator[_T_co]], args: tuple[Any, ...], kwds: dict[str, Any]) -> None: ...
        gen: AsyncGenerator[_T_co, Any]
        func: Callable[..., AsyncGenerator[_T_co, Any]]
        args: tuple[Any, ...]
        kwds: dict[str, Any]

if sys.version_info >= (3, 7):
    def asynccontextmanager(func: Callable[_P, AsyncIterator[_T_co]]) -> Callable[_P, _AsyncGeneratorContextManager[_T_co]]: ...

class _SupportsClose(Protocol):
    def close(self) -> object: ...

_SupportsCloseT = TypeVar("_SupportsCloseT", bound=_SupportsClose)

class closing(AbstractContextManager[_SupportsCloseT]):
    def __init__(self, thing: _SupportsCloseT) -> None: ...

if sys.version_info >= (3, 10):
    class _SupportsAclose(Protocol):
        def aclose(self) -> Awaitable[object]: ...
    _SupportsAcloseT = TypeVar("_SupportsAcloseT", bound=_SupportsAclose)
    class aclosing(AbstractAsyncContextManager[_SupportsAcloseT]):
        def __init__(self, thing: _SupportsAcloseT) -> None: ...

class suppress(AbstractContextManager[None]):
    def __init__(self, *exceptions: Type[BaseException]) -> None: ...
    def __exit__(
        self, exctype: Type[BaseException] | None, excinst: BaseException | None, exctb: TracebackType | None
    ) -> bool: ...

class redirect_stdout(AbstractContextManager[_T_io]):
    def __init__(self, new_target: _T_io) -> None: ...

class redirect_stderr(AbstractContextManager[_T_io]):
    def __init__(self, new_target: _T_io) -> None: ...

class ExitStack(AbstractContextManager[ExitStack]):
    def __init__(self) -> None: ...
    def enter_context(self, cm: AbstractContextManager[_T]) -> _T: ...
    def push(self, exit: _CM_EF) -> _CM_EF: ...
    def callback(self, __callback: Callable[..., Any], *args: Any, **kwds: Any) -> Callable[..., Any]: ...
    def pop_all(self: Self) -> Self: ...
    def close(self) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self, __exc_type: Type[BaseException] | None, __exc_value: BaseException | None, __traceback: TracebackType | None
    ) -> bool: ...

if sys.version_info >= (3, 7):
    _ExitCoroFunc = Callable[[Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]], Awaitable[bool]]
    _CallbackCoroFunc = Callable[..., Awaitable[Any]]
    _ACM_EF = TypeVar("_ACM_EF", AbstractAsyncContextManager[Any], _ExitCoroFunc)
    class AsyncExitStack(AbstractAsyncContextManager[AsyncExitStack]):
        def __init__(self) -> None: ...
        def enter_context(self, cm: AbstractContextManager[_T]) -> _T: ...
        def enter_async_context(self, cm: AbstractAsyncContextManager[_T]) -> Awaitable[_T]: ...
        def push(self, exit: _CM_EF) -> _CM_EF: ...
        def push_async_exit(self, exit: _ACM_EF) -> _ACM_EF: ...
        def callback(self, __callback: Callable[..., Any], *args: Any, **kwds: Any) -> Callable[..., Any]: ...
        def push_async_callback(self, __callback: _CallbackCoroFunc, *args: Any, **kwds: Any) -> _CallbackCoroFunc: ...
        def pop_all(self: Self) -> Self: ...
        def aclose(self) -> Awaitable[None]: ...
        def __aenter__(self: Self) -> Awaitable[Self]: ...
        def __aexit__(
            self, __exc_type: Type[BaseException] | None, __exc_value: BaseException | None, __traceback: TracebackType | None
        ) -> Awaitable[bool]: ...

if sys.version_info >= (3, 10):
    class nullcontext(AbstractContextManager[_T], AbstractAsyncContextManager[_T]):
        enter_result: _T
        @overload
        def __init__(self: nullcontext[None], enter_result: None = ...) -> None: ...
        @overload
        def __init__(self: nullcontext[_T], enter_result: _T) -> None: ...
        def __enter__(self) -> _T: ...
        def __exit__(self, *exctype: Any) -> None: ...
        async def __aenter__(self) -> _T: ...
        async def __aexit__(self, *exctype: Any) -> None: ...

elif sys.version_info >= (3, 7):
    class nullcontext(AbstractContextManager[_T]):
        enter_result: _T
        @overload
        def __init__(self: nullcontext[None], enter_result: None = ...) -> None: ...
        @overload
        def __init__(self: nullcontext[_T], enter_result: _T) -> None: ...
        def __enter__(self) -> _T: ...
        def __exit__(self, *exctype: Any) -> None: ...

if sys.version_info >= (3, 11):
    _T_fd_or_any_path = TypeVar("_T_fd_or_any_path", bound=int | StrOrBytesPath)
    class chdir(AbstractContextManager[None], Generic[_T_fd_or_any_path]):
        path: _T_fd_or_any_path
        def __init__(self, path: _T_fd_or_any_path) -> None: ...
        def __enter__(self) -> None: ...
        def __exit__(self, *excinfo: object) -> None: ...



from typing import Any, Optional, Tuple, Type

from django.db import models
from django.http.request import HttpRequest

SITE_CACHE: Any

class SiteManager(models.Manager["Site"]):
    def get_current(self, request: Optional[HttpRequest] = ...) -> Site: ...
    def clear_cache(self) -> None: ...
    def get_by_natural_key(self, domain: str) -> Site: ...

class Site(models.Model):
    objects: SiteManager

    domain = models.CharField(max_length=100)
    name = models.CharField(max_length=50)
    def natural_key(self) -> Tuple[str]: ...

def clear_site_cache(sender: Type[Site], **kwargs: Any) -> None: ...



from _typeshed import StrOrBytesPath, SupportsRead
from typing import Any, Protocol
from typing_extensions import Literal

LAYOUT_BASIC: Literal[0]
LAYOUT_RAQM: Literal[1]

class _Font(Protocol):
    def getmask(self, text: str | bytes, mode: str = ..., direction=..., features=...): ...

class ImageFont:
    def getsize(self, text: str | bytes, *args, **kwargs) -> tuple[int, int]: ...
    def getmask(self, text: str | bytes, mode: str = ..., direction=..., features=...): ...

class FreeTypeFont:
    path: str | bytes | SupportsRead[bytes] | None
    size: int
    index: int
    encoding: str
    layout_engine: Any
    def __init__(
        self,
        font: str | bytes | SupportsRead[bytes] | None = ...,
        size: int = ...,
        index: int = ...,
        encoding: str = ...,
        layout_engine: int | None = ...,
    ) -> None: ...
    def getname(self) -> tuple[str, str]: ...
    def getmetrics(self) -> tuple[int, int]: ...
    def getlength(
        self,
        text: str | bytes,
        mode: str = ...,
        direction: Literal["ltr", "rtl", "ttb"] | None = ...,
        features: Any | None = ...,
        language: str | None = ...,
    ) -> int: ...
    def getbbox(
        self,
        text: str | bytes,
        mode: str = ...,
        direction=...,
        features=...,
        language: str | None = ...,
        stroke_width: int = ...,
        anchor: str | None = ...,
    ) -> tuple[int, int, int, int]: ...
    def getsize(
        self,
        text: str | bytes,
        direction: Literal["ltr", "rtl", "ttb"] | None = ...,
        features: Any | None = ...,
        language: str | None = ...,
        stroke_width: int = ...,
    ) -> tuple[int, int]: ...
    def getsize_multiline(
        self,
        text: str | bytes,
        direction: Literal["ltr", "rtl", "ttb"] | None = ...,
        spacing: float = ...,
        features: Any | None = ...,
        language: str | None = ...,
        stroke_width: float = ...,
    ) -> tuple[int, int]: ...
    def getoffset(self, text: str | bytes) -> tuple[int, int]: ...
    def getmask(
        self,
        text: str | bytes,
        mode: str = ...,
        direction: Literal["ltr", "rtl", "ttb"] | None = ...,
        features: Any | None = ...,
        language: str | None = ...,
        stroke_width: float = ...,
        anchor: str | None = ...,
        ink=...,
    ): ...
    def getmask2(
        self,
        text: str | bytes,
        mode: str = ...,
        fill=...,
        direction: Literal["ltr", "rtl", "ttb"] | None = ...,
        features: Any | None = ...,
        language: str | None = ...,
        stroke_width: float = ...,
        anchor: str | None = ...,
        ink=...,
        *args,
        **kwargs,
    ): ...
    def font_variant(
        self,
        font: str | bytes | SupportsRead[bytes] | None = ...,
        size: int | None = ...,
        index: int | None = ...,
        encoding: str | None = ...,
        layout_engine: int | None = ...,
    ) -> FreeTypeFont: ...
    def get_variation_names(self): ...
    def set_variation_by_name(self, name): ...
    def get_variation_axes(self): ...
    def set_variation_by_axes(self, axes): ...

class TransposedFont:
    def __init__(self, font: _Font, orientation: int | None = ...) -> None: ...
    def getsize(self, text: str | bytes, *args, **kwargs) -> tuple[int, int]: ...
    def getmask(self, text: str | bytes, mode: str = ..., *args, **kwargs): ...

def load(filename: StrOrBytesPath | int) -> ImageFont: ...
def truetype(
    font: str | bytes | SupportsRead[bytes] | None = ...,
    size: int = ...,
    index: int = ...,
    encoding: str = ...,
    layout_engine: int | None = ...,
) -> FreeTypeFont: ...
def load_path(filename: str | bytes) -> ImageFont: ...
def load_default() -> ImageFont: ...



import sys
from typing import Any, Callable, Optional, Tuple

from ._base import Executor

EXTRA_QUEUED_CALLS: Any

if sys.version_info >= (3, 7):
    from ._base import BrokenExecutor
    class BrokenProcessPool(BrokenExecutor): ...

else:
    class BrokenProcessPool(RuntimeError): ...

if sys.version_info >= (3, 7):
    from multiprocessing.context import BaseContext
    class ProcessPoolExecutor(Executor):
        def __init__(
            self,
            max_workers: Optional[int] = ...,
            mp_context: Optional[BaseContext] = ...,
            initializer: Optional[Callable[..., None]] = ...,
            initargs: Tuple[Any, ...] = ...,
        ) -> None: ...

else:
    class ProcessPoolExecutor(Executor):
        def __init__(self, max_workers: Optional[int] = ...) -> None: ...



import sys
from _typeshed import AnyPath, StrPath
from collections.abc import Callable
from configparser import RawConfigParser
from threading import Thread
from typing import IO, Any, Optional, Pattern, Union

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

if sys.version_info >= (3, 7):
    _Path = AnyPath
else:
    _Path = StrPath

DEFAULT_LOGGING_CONFIG_PORT: int
RESET_ERROR: int  # undocumented
IDENTIFIER: Pattern[str]  # undocumented

def dictConfig(config: dict[str, Any]) -> None: ...

if sys.version_info >= (3, 10):
    def fileConfig(
        fname: Union[_Path, IO[str], RawConfigParser],
        defaults: Optional[dict[str, str]] = ...,
        disable_existing_loggers: bool = ...,
        encoding: Optional[str] = ...,
    ) -> None: ...

else:
    def fileConfig(
        fname: Union[_Path, IO[str], RawConfigParser],
        defaults: Optional[dict[str, str]] = ...,
        disable_existing_loggers: bool = ...,
    ) -> None: ...

def valid_ident(s: str) -> Literal[True]: ...  # undocumented
def listen(port: int = ..., verify: Optional[Callable[[bytes], Optional[bytes]]] = ...) -> Thread: ...
def stopListening() -> None: ...



import sys
from types import CodeType
from typing import IO, Any, Container, Dict, Iterable, Iterator, List, Optional, Sequence, Tuple

LOAD_CONST: int  # undocumented
IMPORT_NAME: int  # undocumented
STORE_NAME: int  # undocumented
STORE_GLOBAL: int  # undocumented
STORE_OPS: Tuple[int, int]  # undocumented
EXTENDED_ARG: int  # undocumented

packagePathMap: Dict[str, List[str]]  # undocumented

def AddPackagePath(packagename: str, path: str) -> None: ...

replacePackageMap: Dict[str, str]  # undocumented

def ReplacePackage(oldname: str, newname: str) -> None: ...

class Module:  # undocumented
    def __init__(self, name: str, file: Optional[str] = ..., path: Optional[str] = ...) -> None: ...
    def __repr__(self) -> str: ...

class ModuleFinder:

    modules: Dict[str, Module]
    path: List[str]  # undocumented
    badmodules: Dict[str, Dict[str, int]]  # undocumented
    debug: int  # undocumented
    indent: int  # undocumented
    excludes: Container[str]  # undocumented
    replace_paths: Sequence[Tuple[str, str]]  # undocumented

    if sys.version_info >= (3, 8):
        def __init__(
            self,
            path: Optional[List[str]] = ...,
            debug: int = ...,
            excludes: Optional[Container[str]] = ...,
            replace_paths: Optional[Sequence[Tuple[str, str]]] = ...,
        ) -> None: ...
    else:
        def __init__(
            self,
            path: Optional[List[str]] = ...,
            debug: int = ...,
            excludes: Container[str] = ...,
            replace_paths: Sequence[Tuple[str, str]] = ...,
        ) -> None: ...
    def msg(self, level: int, str: str, *args: Any) -> None: ...  # undocumented
    def msgin(self, *args: Any) -> None: ...  # undocumented
    def msgout(self, *args: Any) -> None: ...  # undocumented
    def run_script(self, pathname: str) -> None: ...
    def load_file(self, pathname: str) -> None: ...  # undocumented
    def import_hook(
        self, name: str, caller: Optional[Module] = ..., fromlist: Optional[List[str]] = ..., level: int = ...
    ) -> Optional[Module]: ...  # undocumented
    def determine_parent(self, caller: Optional[Module], level: int = ...) -> Optional[Module]: ...  # undocumented
    def find_head_package(self, parent: Module, name: str) -> Tuple[Module, str]: ...  # undocumented
    def load_tail(self, q: Module, tail: str) -> Module: ...  # undocumented
    def ensure_fromlist(self, m: Module, fromlist: Iterable[str], recursive: int = ...) -> None: ...  # undocumented
    def find_all_submodules(self, m: Module) -> Iterable[str]: ...  # undocumented
    def import_module(self, partname: str, fqname: str, parent: Module) -> Optional[Module]: ...  # undocumented
    def load_module(self, fqname: str, fp: IO[str], pathname: str, file_info: Tuple[str, str, str]) -> Module: ...  # undocumented
    if sys.version_info >= (3, 6):
        def scan_opcodes(self, co: CodeType) -> Iterator[Tuple[str, Tuple[Any, ...]]]: ...  # undocumented
    def scan_code(self, co: CodeType, m: Module) -> None: ...  # undocumented
    def load_package(self, fqname: str, pathname: str) -> Module: ...  # undocumented
    def add_module(self, fqname: str) -> Module: ...  # undocumented
    def find_module(
        self, name: str, path: Optional[str], parent: Optional[Module] = ...
    ) -> Tuple[Optional[IO[Any]], Optional[str], Tuple[str, str, int]]: ...  # undocumented
    def report(self) -> None: ...
    def any_missing(self) -> List[str]: ...  # undocumented
    def any_missing_maybe(self) -> Tuple[List[str], List[str]]: ...  # undocumented
    def replace_paths_in_code(self, co: CodeType) -> CodeType: ...  # undocumented



import copy
from typing import List, Optional, Type

from django.db.models.base import Model

from nwon_django_toolbox.tests.api.api_test import ApiTest
from nwon_django_toolbox.tests.helper.check_object_against_parameter import (
    check_object_against_parameter,
)
from nwon_django_toolbox.typings import RequestBodyFormat
from nwon_django_toolbox.url_helper import (
    detail_url_for_model,
    detail_url_for_model_class,
    list_url_for_model_class,
)


def check_post_basics(
    model_class: Type[Model],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    api_test = ApiTest()
    url = detail_url_for_model_class(model_class, "some-non-existing-id")

    # post on detail url is not allowed
    api_test.post_unauthorized(url, {}, body_format)

    if authentication_token:
        api_test.set_bearer_token(authentication_token)
        api_test.post_method_not_allowed(url, {}, body_format)


def check_post_not_allowed(
    model_class: Type[Model],
    authentication_token: Optional[str] = None,
):
    list_url = list_url_for_model_class(model_class)

    api_test = ApiTest(token=authentication_token)
    api_test.post_method_not_allowed(list_url, {})


def check_post_parameters_successful(
    model_class: Type[Model],
    successful_parameters: List[dict],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    for successful_parameter in successful_parameters:
        # test creation of object
        created_object = api_test.post_create_successful(
            list_url, successful_parameter, body_format=body_format
        )
        check_object_against_parameter(created_object, successful_parameter)

        # get object via get
        created_objet = model_class.objects.get(pk=created_object["id"])
        url = detail_url_for_model(created_objet)
        object_to_check = api_test.get_successful(url)
        check_object_against_parameter(object_to_check, successful_parameter)


def check_post_parameters_required(
    model_class: Type[Model],
    successful_parameter: dict,
    required_keys: List[str],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    # make sure that parameter are working
    api_test.post_create_successful(
        list_url, successful_parameter, body_format=body_format
    )

    for key in required_keys:
        parameter = copy.deepcopy(successful_parameter)
        parameter.pop(key)
        api_test.post_bad_request(list_url, parameter, body_format=body_format)


def check_post_parameters_not_required(
    model_class: Type[Model],
    successful_parameter: dict,
    required_keys: List[str],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    # make sure that parameter are working
    api_test.post_create_successful(
        list_url, successful_parameter, body_format=body_format
    )

    for key in required_keys:
        parameter = copy.deepcopy(successful_parameter)
        parameter.pop(key)
        api_test.post_create_successful(list_url, parameter, body_format=body_format)


def check_post_parameters_failing(
    model_class: Type[Model],
    failing_parameters: List[dict],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    for failing_parameter in failing_parameters:
        api_test.post_bad_request(list_url, failing_parameter, body_format=body_format)


def check_post_read_only_field(
    model_class: Type[Model],
    successful_post_parameter: dict,
    key: str,
    value,
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    successful_post_parameter[key] = value

    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    created_object = api_test.post_create_successful(
        list_url, successful_post_parameter, body_format=body_format
    )

    # check whether read only key was set
    assert created_object[key] != value


__all__ = [
    "check_post_basics",
    "check_post_not_allowed",
    "check_post_parameters_failing",
    "check_post_parameters_successful",
    "check_post_read_only_field",
    "check_post_parameters_not_required",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nwon_django_toolbox',
 'nwon_django_toolbox.admin',
 'nwon_django_toolbox.exception_handler',
 'nwon_django_toolbox.fields',
 'nwon_django_toolbox.fields.encrypted_text_field',
 'nwon_django_toolbox.fields.pydantic_json_field',
 'nwon_django_toolbox.files',
 'nwon_django_toolbox.fixtures',
 'nwon_django_toolbox.tests',
 'nwon_django_toolbox.tests.api',
 'nwon_django_toolbox.tests.api_helper',
 'nwon_django_toolbox.tests.celery',
 'nwon_django_toolbox.tests.helper',
 'nwon_django_toolbox.typings',
 'nwon_django_toolbox.typings.celery',
 'nwon_django_toolbox.url_helper',
 'nwon_django_toolbox.yasg']

package_data = \
{'': ['*']}

install_requires = \
['Pillow-PIL>=0.1dev,<0.2',
 'django-json-widget>=1.1.1,<2.0.0',
 'django-polymorphic>=3.1.0,<4.0.0',
 'django-rest-polymorphic>=0.1.9,<0.2.0',
 'django>=3.0',
 'drf-spectacular>=0.22.0,<0.23.0',
 'drf-yasg>=1.20.0,<2.0.0',
 'jsonref>=0.2,<0.3',
 'jsonschema-to-openapi>=0.2.1,<0.3.0',
 'nwon-baseline>=0.1.19,<0.2.0',
 'pydantic>=1.8.2',
 'pyhumps>=3.0.2,<4.0.0']

entry_points = \
{'console_scripts': ['prepare = scripts.prepare:prepare']}

setup_kwargs = {
    'name': 'nwon-django-toolbox',
    'version': '0.2.23',
    'description': 'Some Django Wizardry',
    'long_description': '# NWON-Django-Toolbox\n\nThis package provides some Django additions that can be used across several projects.\n\n## Settings\n\nThe Django Toolbox can be configured using the Django settings. We expect the key `NWON_DJANGO` that holds a dictionary. The dictionary must be of type `NWONDjangoSettings` that comes with this package (`nwon_django_toolbox.nwon_django_settings`). The keys mus be snake case or camel case.\n\nFor example like this\n\n```python\nNWON_DJANGO: NWONDjangoSettings = {\n    "authorization_prefix": "Bearer",\n    "logger_name": "your-log-name",\n    "application_name": "application"\n}\n```\n\n## Dependencies\n\nThe project has a bunch of dependencies that we use in most of our projects. In the end we have quite a lot and need to slim this down in the future.\n\nDjango related libraries are:\n\n- Django (Obviously 🧠)\n- django-polymorphic\n- django-json-widget\n- django-rest-polymorphic\n\nFor API documentation our models and serializer support two library which come as a dependency as well:\n\n- drf-spectacular\n- drf-yasg\n\nOn top we use a bunch of helper libraries\n\n- Pydantic\n- Pillow-PIL\n- jsonref\n- jsonschema-to-openapi\n- pyhumps\n- nwon-baseline\n\nPackage is meant for internal use at [NWON](https://nwon.de) as breaking changes may occur on version changes. This may change at some point but not for now 😇.\n\n## Development Setup\n\nWe recommend developing using poetry.\n\nThis are the steps to setup the project with a local virtual environment:\n\n1. Tell poetry to create dependencies in a `.venv` folder withing the project: `poetry config virtualenvs.in-project true`\n1. Create a virtual environment using the local python version: `poetry env use $(cat .python-version)`\n1. Install dependencies: `poetry install`\n\n## Prepare Package\n\nBefore publishing the package we need to:\n\n1. Clean dist folder\n1. Bump up the version of the package\n1. Build the package\n\nLuckily we provide a script for doing all of this `python scripts/prepare.py patch`. Alternatively you can run the script in a poetry context `poetry run prepare patch`. The argument at the end defines whether you want a `patch`, `minor` or `major` version bump.\n\nThe final zipped data ends up in the `dist` folder.\n\n## Publish Package \n\nTest package publication\n\n1. Add test PyPi repository: `poetry config repositories.testpypi https://test.pypi.org/legacy/`\n2. Publish the package to the test repository: `poetry publish -r testpypi`\n3. Test package: `pip install --index-url https://test.pypi.org/simple/ nwon_baseline`\n\nIf everything works fine publish the package via `poetry publish`.\n',
    'author': 'Reik Stiebeling',
    'author_email': 'reik.stiebeling@nwon.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://nwon.de',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Iterable

from django.db.backends.base.base import BaseDatabaseWrapper

class BaseDatabaseClient:
    executable_name: Any = ...
    connection: BaseDatabaseWrapper
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    def runshell(self, parameters: Iterable[str]) -> None: ...



from typing import Iterable, List, Optional, Pattern, Union, overload
from typing_extensions import Literal

# class is entirely undocumented
class FileList:
    allfiles: Optional[Iterable[str]] = ...
    files: List[str] = ...
    def __init__(self, warn: None = ..., debug_print: None = ...) -> None: ...
    def set_allfiles(self, allfiles: Iterable[str]) -> None: ...
    def findall(self, dir: str = ...) -> None: ...
    def debug_print(self, msg: str) -> None: ...
    def append(self, item: str) -> None: ...
    def extend(self, items: Iterable[str]) -> None: ...
    def sort(self) -> None: ...
    def remove_duplicates(self) -> None: ...
    def process_template_line(self, line: str) -> None: ...
    @overload
    def include_pattern(
        self, pattern: str, anchor: Union[int, bool] = ..., prefix: Optional[str] = ..., is_regex: Literal[0, False] = ...
    ) -> bool: ...
    @overload
    def include_pattern(self, pattern: Union[str, Pattern[str]], *, is_regex: Literal[True, 1] = ...) -> bool: ...
    @overload
    def include_pattern(
        self,
        pattern: Union[str, Pattern[str]],
        anchor: Union[int, bool] = ...,
        prefix: Optional[str] = ...,
        is_regex: Union[int, bool] = ...,
    ) -> bool: ...
    @overload
    def exclude_pattern(
        self, pattern: str, anchor: Union[int, bool] = ..., prefix: Optional[str] = ..., is_regex: Literal[0, False] = ...
    ) -> bool: ...
    @overload
    def exclude_pattern(self, pattern: Union[str, Pattern[str]], *, is_regex: Literal[True, 1] = ...) -> bool: ...
    @overload
    def exclude_pattern(
        self,
        pattern: Union[str, Pattern[str]],
        anchor: Union[int, bool] = ...,
        prefix: Optional[str] = ...,
        is_regex: Union[int, bool] = ...,
    ) -> bool: ...

def findall(dir: str = ...) -> List[str]: ...
def glob_to_re(pattern: str) -> str: ...
@overload
def translate_pattern(
    pattern: str, anchor: Union[int, bool] = ..., prefix: Optional[str] = ..., is_regex: Literal[False, 0] = ...
) -> Pattern[str]: ...
@overload
def translate_pattern(pattern: Union[str, Pattern[str]], *, is_regex: Literal[True, 1] = ...) -> Pattern[str]: ...
@overload
def translate_pattern(
    pattern: Union[str, Pattern[str]],
    anchor: Union[int, bool] = ...,
    prefix: Optional[str] = ...,
    is_regex: Union[int, bool] = ...,
) -> Pattern[str]: ...



import sys
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Type, Union

from django.apps.config import AppConfig
from django.contrib.admin.options import ModelAdmin
from django.contrib.auth.forms import AuthenticationForm
from django.core.checks import CheckMessage
from django.core.handlers.wsgi import WSGIRequest
from django.db.models.base import Model
from django.db.models.query import QuerySet
from django.http.response import HttpResponse
from django.template.response import TemplateResponse
from django.urls import URLPattern, URLResolver
from django.utils.functional import LazyObject

if sys.version_info >= (3, 9):
    from weakref import WeakSet

    all_sites: WeakSet[AdminSite]
else:
    from typing import MutableSet

    all_sites: MutableSet[AdminSite]

_ActionCallback = Callable[[ModelAdmin, WSGIRequest, QuerySet], Optional[TemplateResponse]]

class AlreadyRegistered(Exception): ...
class NotRegistered(Exception): ...

class AdminSite:
    site_title: str = ...
    site_header: str = ...
    index_title: str = ...
    site_url: str = ...
    login_form: Optional[Type[AuthenticationForm]] = ...
    index_template: Optional[str] = ...
    app_index_template: Optional[str] = ...
    login_template: Optional[str] = ...
    logout_template: Optional[str] = ...
    password_change_template: Optional[str] = ...
    password_change_done_template: Optional[str] = ...
    name: str = ...
    enable_nav_sidebar: bool = ...
    final_catch_all_view: bool = ...
    _empty_value_display: str = ...
    _registry: Dict[Type[Model], ModelAdmin]
    _global_actions: Dict[str, _ActionCallback]
    _actions: Dict[str, _ActionCallback]
    def __init__(self, name: str = ...) -> None: ...
    def check(self, app_configs: Optional[Iterable[AppConfig]]) -> List[CheckMessage]: ...
    def register(
        self,
        model_or_iterable: Union[Type[Model], Iterable[Type[Model]]],
        admin_class: Optional[Type[ModelAdmin]] = ...,
        **options: Any
    ) -> None: ...
    def unregister(self, model_or_iterable: Union[Type[Model], Iterable[Type[Model]]]) -> None: ...
    def is_registered(self, model: Type[Model]) -> bool: ...
    def add_action(self, action: _ActionCallback, name: Optional[str] = ...) -> None: ...
    def disable_action(self, name: str) -> None: ...
    def get_action(self, name: str) -> Callable: ...
    @property
    def actions(self) -> Iterable[Tuple[str, _ActionCallback]]: ...
    @property
    def empty_value_display(self) -> str: ...
    @empty_value_display.setter
    def empty_value_display(self, empty_value_display: str) -> None: ...
    def has_permission(self, request: WSGIRequest) -> bool: ...
    def admin_view(self, view: Callable, cacheable: bool = ...) -> Callable: ...
    def get_urls(self) -> List[URLResolver]: ...
    @property
    def urls(self) -> Tuple[List[Union[URLResolver, URLPattern]], str, str]: ...
    def each_context(self, request: WSGIRequest) -> Dict[str, Any]: ...
    def password_change(
        self, request: WSGIRequest, extra_context: Optional[Dict[str, Any]] = ...
    ) -> TemplateResponse: ...
    def password_change_done(
        self, request: WSGIRequest, extra_context: Optional[Dict[str, Any]] = ...
    ) -> TemplateResponse: ...
    def i18n_javascript(self, request: WSGIRequest, extra_context: Optional[Dict[str, Any]] = ...) -> HttpResponse: ...
    def logout(self, request: WSGIRequest, extra_context: Optional[Dict[str, Any]] = ...) -> TemplateResponse: ...
    def login(self, request: WSGIRequest, extra_context: Optional[Dict[str, Any]] = ...) -> HttpResponse: ...
    def _build_app_dict(self, request: WSGIRequest, label: Optional[str] = ...) -> Dict[str, Any]: ...
    def get_app_list(self, request: WSGIRequest) -> List[Any]: ...
    def index(self, request: WSGIRequest, extra_context: Optional[Dict[str, Any]] = ...) -> TemplateResponse: ...
    def app_index(
        self, request: WSGIRequest, app_label: str, extra_context: Optional[Dict[str, Any]] = ...
    ) -> TemplateResponse: ...

class DefaultAdminSite(LazyObject): ...

site: AdminSite



from typing import Any, Callable, Dict, List, Optional, Union

from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.utils.translation.trans_real import DjangoTranslation
from django.views.generic import View

LANGUAGE_QUERY_PARAMETER: str

def set_language(request: HttpRequest) -> HttpResponse: ...
def get_formats() -> Dict[str, Union[List[str], int, str]]: ...

js_catalog_template: str

def render_javascript_catalog(catalog: Optional[Any] = ..., plural: Optional[Any] = ...): ...
def null_javascript_catalog(request: Any, domain: Optional[Any] = ..., packages: Optional[Any] = ...): ...

class JavaScriptCatalog(View):
    head: Callable
    domain: str = ...
    packages: List[str] = ...
    translation: DjangoTranslation = ...
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def get_paths(self, packages: List[str]) -> List[str]: ...
    def get_plural(self) -> None: ...
    def get_catalog(self) -> Dict[str, Union[List[str], str]]: ...
    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]: ...
    def render_to_response(self, context: Dict[str, Any], **response_kwargs: Any) -> HttpResponse: ...

class JSONCatalog(JavaScriptCatalog): ...



import os
from typing import List, Tuple, Union

from django.core.management.base import BaseCommand as BaseCommand
from django.core.management.base import CommandError as CommandError
from django.core.management.base import CommandParser as CommandParser
from django.core.management.utils import find_command as find_command
from django.core.management.utils import popen_wrapper as popen_wrapper

_PathType = Union[str, bytes, os.PathLike]

def has_bom(fn: _PathType) -> bool: ...
def is_writable(path: _PathType) -> bool: ...

class Command(BaseCommand):
    program: str = ...
    program_options: List[str] = ...
    verbosity: int = ...
    has_errors: bool = ...
    def compile_messages(self, locations: List[Tuple[_PathType, _PathType]]) -> None: ...



from typing import Any, ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

class GifImageFile(ImageFile):
    format: ClassVar[Literal["GIF"]]
    format_description: ClassVar[str]
    global_palette: Any
    def data(self): ...
    @property
    def n_frames(self): ...
    @property
    def is_animated(self): ...
    im: Any
    def seek(self, frame) -> None: ...
    def tell(self): ...

RAWMODE: Any

def get_interlace(im): ...
def getheader(im, palette: Any | None = ..., info: Any | None = ...): ...
def getdata(im, offset=..., **params): ...



from datetime import datetime
from typing import Any, Dict, Optional, Type

from django.contrib.sessions.backends.base import SessionBase
from django.db import models

class BaseSessionManager(models.Manager):
    def encode(self, session_dict: Dict[str, int]) -> str: ...
    def save(self, session_key: str, session_dict: Dict[str, int], expire_date: datetime) -> AbstractBaseSession: ...

class AbstractBaseSession(models.Model):
    expire_date: datetime
    session_data: str
    session_key: str
    objects: Any = ...
    @classmethod
    def get_session_store_class(cls) -> Optional[Type[SessionBase]]: ...
    def get_decoded(self) -> Dict[str, int]: ...



import sys
from typing import List, NamedTuple

if sys.platform != "win32":
    class struct_group(NamedTuple):
        gr_name: str
        gr_passwd: str | None
        gr_gid: int
        gr_mem: List[str]
    def getgrall() -> List[struct_group]: ...
    def getgrgid(id: int) -> struct_group: ...
    def getgrnam(name: str) -> struct_group: ...



from typing import Any, Dict, Iterable, Iterator, List, Optional, Type

from django.apps import AppConfig
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Model

DEFAULT_DB_ALIAS: str
DJANGO_VERSION_PICKLE_KEY: str

class Error(Exception): ...
class InterfaceError(Error): ...
class DatabaseError(Error): ...
class DataError(DatabaseError): ...
class OperationalError(DatabaseError): ...
class IntegrityError(DatabaseError): ...
class InternalError(DatabaseError): ...
class ProgrammingError(DatabaseError): ...
class NotSupportedError(DatabaseError): ...

def load_backend(backend_name: str) -> Any: ...

class ConnectionDoesNotExist(Exception): ...

class ConnectionHandler:
    databases: Dict[str, Dict[str, Optional[Any]]]
    def __init__(self, databases: Dict[str, Dict[str, Optional[Any]]] = ...) -> None: ...
    def ensure_defaults(self, alias: str) -> None: ...
    def prepare_test_settings(self, alias: str) -> None: ...
    def __getitem__(self, alias: str) -> BaseDatabaseWrapper: ...
    def __setitem__(self, key: str, value: BaseDatabaseWrapper) -> None: ...
    def __delitem__(self, key: BaseDatabaseWrapper) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def all(self) -> List[BaseDatabaseWrapper]: ...
    def close_all(self) -> None: ...

class ConnectionRouter:
    def __init__(self, routers: Optional[Iterable[Any]] = ...) -> None: ...
    @property
    def routers(self) -> List[Any]: ...
    def db_for_read(self, model: Type[Model], **hints: Any) -> str: ...
    def db_for_write(self, model: Type[Model], **hints: Any) -> str: ...
    def allow_relation(self, obj1: Model, obj2: Model, **hints: Any) -> bool: ...
    def allow_migrate(self, db: str, app_label: str, **hints: Any) -> bool: ...
    def allow_migrate_model(self, db: str, model: Type[Model]) -> bool: ...
    def get_migratable_models(
        self, app_config: AppConfig, db: str, include_auto_created: bool = ...
    ) -> List[Type[Model]]: ...



from typing import Any, List

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.fields import Field

class BaseDatabaseValidation:
    connection: BaseDatabaseWrapper = ...
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    def check(self, **kwargs: Any) -> List[Any]: ...
    def check_field(self, field: Field, **kwargs: Any) -> List[Any]: ...



import io
from _typeshed import ReadableBuffer, Self, StrOrBytesPath
from typing import IO, Any, Mapping, Sequence, TextIO, Union, overload
from typing_extensions import Literal, final

_OpenBinaryWritingMode = Literal["w", "wb", "x", "xb", "a", "ab"]
_OpenTextWritingMode = Literal["wt", "xt", "at"]

_PathOrFile = Union[StrOrBytesPath, IO[bytes]]

_FilterChain = Sequence[Mapping[str, Any]]

FORMAT_AUTO: int
FORMAT_XZ: int
FORMAT_ALONE: int
FORMAT_RAW: int
CHECK_NONE: int
CHECK_CRC32: int
CHECK_CRC64: int
CHECK_SHA256: int
CHECK_ID_MAX: int
CHECK_UNKNOWN: int
FILTER_LZMA1: int
FILTER_LZMA2: int
FILTER_DELTA: int
FILTER_X86: int
FILTER_IA64: int
FILTER_ARM: int
FILTER_ARMTHUMB: int
FILTER_SPARC: int
FILTER_POWERPC: int
MF_HC3: int
MF_HC4: int
MF_BT2: int
MF_BT3: int
MF_BT4: int
MODE_FAST: int
MODE_NORMAL: int
PRESET_DEFAULT: int
PRESET_EXTREME: int

# from _lzma.c
@final
class LZMADecompressor:
    def __init__(self, format: int | None = ..., memlimit: int | None = ..., filters: _FilterChain | None = ...) -> None: ...
    def decompress(self, data: bytes, max_length: int = ...) -> bytes: ...
    @property
    def check(self) -> int: ...
    @property
    def eof(self) -> bool: ...
    @property
    def unused_data(self) -> bytes: ...
    @property
    def needs_input(self) -> bool: ...

# from _lzma.c
@final
class LZMACompressor:
    def __init__(
        self, format: int | None = ..., check: int = ..., preset: int | None = ..., filters: _FilterChain | None = ...
    ) -> None: ...
    def compress(self, __data: bytes) -> bytes: ...
    def flush(self) -> bytes: ...

class LZMAError(Exception): ...

class LZMAFile(io.BufferedIOBase, IO[bytes]):
    def __init__(
        self,
        filename: _PathOrFile | None = ...,
        mode: str = ...,
        *,
        format: int | None = ...,
        check: int = ...,
        preset: int | None = ...,
        filters: _FilterChain | None = ...,
    ) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def close(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    def fileno(self) -> int: ...
    def seekable(self) -> bool: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def peek(self, size: int = ...) -> bytes: ...
    def read(self, size: int | None = ...) -> bytes: ...
    def read1(self, size: int = ...) -> bytes: ...
    def readline(self, size: int | None = ...) -> bytes: ...
    def write(self, data: ReadableBuffer) -> int: ...
    def seek(self, offset: int, whence: int = ...) -> int: ...
    def tell(self) -> int: ...

@overload
def open(
    filename: _PathOrFile,
    mode: Literal["r", "rb"] = ...,
    *,
    format: int | None = ...,
    check: Literal[-1] = ...,
    preset: None = ...,
    filters: _FilterChain | None = ...,
    encoding: None = ...,
    errors: None = ...,
    newline: None = ...,
) -> LZMAFile: ...
@overload
def open(
    filename: _PathOrFile,
    mode: _OpenBinaryWritingMode,
    *,
    format: int | None = ...,
    check: int = ...,
    preset: int | None = ...,
    filters: _FilterChain | None = ...,
    encoding: None = ...,
    errors: None = ...,
    newline: None = ...,
) -> LZMAFile: ...
@overload
def open(
    filename: StrOrBytesPath,
    mode: Literal["rt"],
    *,
    format: int | None = ...,
    check: Literal[-1] = ...,
    preset: None = ...,
    filters: _FilterChain | None = ...,
    encoding: str | None = ...,
    errors: str | None = ...,
    newline: str | None = ...,
) -> TextIO: ...
@overload
def open(
    filename: StrOrBytesPath,
    mode: _OpenTextWritingMode,
    *,
    format: int | None = ...,
    check: int = ...,
    preset: int | None = ...,
    filters: _FilterChain | None = ...,
    encoding: str | None = ...,
    errors: str | None = ...,
    newline: str | None = ...,
) -> TextIO: ...
@overload
def open(
    filename: _PathOrFile,
    mode: str,
    *,
    format: int | None = ...,
    check: int = ...,
    preset: int | None = ...,
    filters: _FilterChain | None = ...,
    encoding: str | None = ...,
    errors: str | None = ...,
    newline: str | None = ...,
) -> LZMAFile | TextIO: ...
def compress(
    data: bytes, format: int = ..., check: int = ..., preset: int | None = ..., filters: _FilterChain | None = ...
) -> bytes: ...
def decompress(data: bytes, format: int = ..., memlimit: int | None = ..., filters: _FilterChain | None = ...) -> bytes: ...
def is_check_supported(__check_id: int) -> bool: ...



from django.apps import apps as apps
from django.conf import settings as settings
from django.core.management.base import BaseCommand as BaseCommand
from django.core.management.base import CommandError as CommandError
from django.db import DEFAULT_DB_ALIAS as DEFAULT_DB_ALIAS
from django.db import connections as connections
from django.db import migrations as migrations
from django.db.migrations.loader import AmbiguityError as AmbiguityError
from django.db.migrations.loader import MigrationLoader as MigrationLoader
from django.db.migrations.migration import Migration
from django.db.migrations.migration import SwappableTuple as SwappableTuple
from django.db.migrations.optimizer import MigrationOptimizer as MigrationOptimizer
from django.db.migrations.writer import MigrationWriter as MigrationWriter
from django.utils.version import get_docs_version as get_docs_version

class Command(BaseCommand):
    verbosity: int = ...
    interactive: bool = ...
    def find_migration(self, loader: MigrationLoader, app_label: str, name: str) -> Migration: ...



from typing import AnyStr, Dict, List, NamedTuple, Sequence, Tuple, Union, overload

_String = Union[str, unicode]

uses_relative: List[str]
uses_netloc: List[str]
uses_params: List[str]
non_hierarchical: List[str]
uses_query: List[str]
uses_fragment: List[str]
scheme_chars: str
MAX_CACHE_SIZE: int

def clear_cache() -> None: ...

class ResultMixin(object):
    @property
    def username(self) -> str | None: ...
    @property
    def password(self) -> str | None: ...
    @property
    def hostname(self) -> str | None: ...
    @property
    def port(self) -> int | None: ...

class _SplitResult(NamedTuple):
    scheme: str
    netloc: str
    path: str
    query: str
    fragment: str

class SplitResult(_SplitResult, ResultMixin):
    def geturl(self) -> str: ...

class _ParseResult(NamedTuple):
    scheme: str
    netloc: str
    path: str
    params: str
    query: str
    fragment: str

class ParseResult(_ParseResult, ResultMixin):
    def geturl(self) -> _String: ...

def urlparse(url: _String, scheme: _String = ..., allow_fragments: bool = ...) -> ParseResult: ...
def urlsplit(url: _String, scheme: _String = ..., allow_fragments: bool = ...) -> SplitResult: ...
@overload
def urlunparse(data: Tuple[AnyStr, AnyStr, AnyStr, AnyStr, AnyStr, AnyStr]) -> AnyStr: ...
@overload
def urlunparse(data: Sequence[AnyStr]) -> AnyStr: ...
@overload
def urlunsplit(data: Tuple[AnyStr, AnyStr, AnyStr, AnyStr, AnyStr]) -> AnyStr: ...
@overload
def urlunsplit(data: Sequence[AnyStr]) -> AnyStr: ...
def urljoin(base: AnyStr, url: AnyStr, allow_fragments: bool = ...) -> AnyStr: ...
def urldefrag(url: AnyStr) -> Tuple[AnyStr, AnyStr]: ...
def unquote(s: AnyStr) -> AnyStr: ...
def parse_qs(qs: AnyStr, keep_blank_values: bool = ..., strict_parsing: bool = ...) -> Dict[AnyStr, List[AnyStr]]: ...
def parse_qsl(qs: AnyStr, keep_blank_values: int = ..., strict_parsing: bool = ...) -> List[Tuple[AnyStr, AnyStr]]: ...



from typing import Any

from django.contrib import admin
from django.core.handlers.wsgi import WSGIRequest
from django.http.response import HttpResponse

csrf_protect_m: Any
sensitive_post_parameters_m: Any

class GroupAdmin(admin.ModelAdmin): ...

class UserAdmin(admin.ModelAdmin):
    change_user_password_template: Any = ...
    add_fieldsets: Any = ...
    add_form: Any = ...
    change_password_form: Any = ...
    def user_change_password(self, request: WSGIRequest, id: str, form_url: str = ...) -> HttpResponse: ...



from django import forms
from django.contrib import admin
from django.db import models
from django_json_widget.widgets import JSONEditorWidget
from polymorphic.admin import PolymorphicChildModelAdmin, PolymorphicParentModelAdmin

from nwon_django_toolbox.fields import PydanticJsonField
from nwon_django_toolbox.fields.encrypted_text_field.encrypted_text_field import (
    EncryptedTextField,
)


class PolymorphicParentModelAdminWithJsonEditor(PolymorphicParentModelAdmin):
    formfield_overrides = {
        models.JSONField: {"widget": JSONEditorWidget},
        PydanticJsonField: {"widget": JSONEditorWidget},
    }


class PolymorphicChildModelAdminWithJsonEditor(PolymorphicChildModelAdmin):
    formfield_overrides = {
        models.JSONField: {"widget": JSONEditorWidget},
        PydanticJsonField: {"widget": JSONEditorWidget},
    }


class ModelAdminWithJsonEditor(admin.ModelAdmin):
    formfield_overrides = {
        models.JSONField: {"widget": JSONEditorWidget},
        PydanticJsonField: {"widget": JSONEditorWidget},
    }


class ModelAdminWithJsonEditorReadOnly(admin.ModelAdmin):
    formfield_overrides = {
        models.JSONField: {
            "widget": JSONEditorWidget(options={"mode": "view", "modes": ["view"]})
        },
        PydanticJsonField: {
            "widget": JSONEditorWidget(options={"mode": "view", "modes": ["view"]})
        },
        EncryptedTextField: {
            "widget": forms.PasswordInput(render_value=False),
        },
    }


__all__ = [
    "ModelAdminWithJsonEditorReadOnly",
    "ModelAdminWithJsonEditor",
    "PolymorphicParentModelAdminWithJsonEditor",
    "PolymorphicChildModelAdminWithJsonEditor",
]



from django.contrib.gis.db.backends.base.features import BaseSpatialFeatures as BaseSpatialFeatures
from django.db.backends.sqlite3.features import DatabaseFeatures as SQLiteDatabaseFeatures

class DatabaseFeatures(BaseSpatialFeatures, SQLiteDatabaseFeatures):
    supports_3d_storage: bool = ...
    def supports_area_geodetic(self): ...



from .api import MessageFailure as MessageFailure
from .api import add_message as add_message
from .api import debug as debug
from .api import error as error
from .api import get_level as get_level
from .api import get_messages as get_messages
from .api import info as info
from .api import set_level as set_level
from .api import success as success
from .api import warning as warning
from .constants import DEBUG as DEBUG
from .constants import DEFAULT_LEVELS as DEFAULT_LEVELS
from .constants import DEFAULT_TAGS as DEFAULT_TAGS
from .constants import ERROR as ERROR
from .constants import INFO as INFO
from .constants import SUCCESS as SUCCESS
from .constants import WARNING as WARNING

default_app_config: str = ...



from typing import Any, Callable, Dict, Generic, Optional, Sequence, Type, TypeVar, Union

from django.db import models
from django.forms.forms import BaseForm
from django.forms.models import BaseModelForm
from django.http import HttpRequest, HttpResponse
from django.views.generic.base import ContextMixin, TemplateResponseMixin, View
from django.views.generic.detail import BaseDetailView, SingleObjectMixin, SingleObjectTemplateResponseMixin
from typing_extensions import Literal

_FormT = TypeVar("_FormT", bound=BaseForm)
_ModelFormT = TypeVar("_ModelFormT", bound=BaseModelForm)
_T = TypeVar("_T", bound=models.Model)

class AbstractFormMixin(Generic[_FormT], ContextMixin):
    initial: Dict[str, Any] = ...
    form_class: Optional[Type[_FormT]] = ...
    success_url: Optional[Union[str, Callable[..., Any]]] = ...
    prefix: Optional[str] = ...
    def get_initial(self) -> Dict[str, Any]: ...
    def get_prefix(self) -> Optional[str]: ...
    def get_form_kwargs(self) -> Dict[str, Any]: ...
    def get_success_url(self) -> str: ...

class FormMixin(Generic[_FormT], AbstractFormMixin):
    def get_form_class(self) -> Type[_FormT]: ...
    def get_form(self, form_class: Optional[Type[_FormT]] = ...) -> BaseForm: ...
    def form_valid(self, form: _FormT) -> HttpResponse: ...
    def form_invalid(self, form: _FormT) -> HttpResponse: ...

class ModelFormMixin(Generic[_T, _ModelFormT], AbstractFormMixin, SingleObjectMixin[_T]):
    fields: Optional[Union[Sequence[str], Literal["__all__"]]] = ...
    def get_form_class(self) -> Type[_ModelFormT]: ...
    def get_form(self, form_class: Optional[Type[_ModelFormT]] = ...) -> BaseModelForm: ...
    def form_valid(self, form: _ModelFormT) -> HttpResponse: ...
    def form_invalid(self, form: _ModelFormT) -> HttpResponse: ...

class ProcessFormView(View):
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def put(self, *args: Any, **kwargs: Any) -> HttpResponse: ...

class BaseFormView(FormMixin[_FormT], ProcessFormView): ...
class FormView(TemplateResponseMixin, BaseFormView[_FormT]): ...
class BaseCreateView(ModelFormMixin[_T, _ModelFormT], ProcessFormView): ...
class CreateView(SingleObjectTemplateResponseMixin, BaseCreateView[_T, _ModelFormT]): ...
class BaseUpdateView(ModelFormMixin[_T, _ModelFormT], ProcessFormView): ...
class UpdateView(SingleObjectTemplateResponseMixin, BaseUpdateView[_T, _ModelFormT]): ...

class DeletionMixin:
    success_url: Optional[str] = ...
    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def delete(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse: ...
    def get_success_url(self) -> str: ...

class BaseDeleteView(DeletionMixin, BaseDetailView): ...
class DeleteView(SingleObjectTemplateResponseMixin, BaseDeleteView): ...



from functools import wraps as wraps  # noqa: F401
from typing import Any, Callable, Dict, Generic, List, Optional, Tuple, Type, TypeVar, Union, overload

from django.db.models.base import Model

def curry(_curried_func: Any, *args: Any, **kwargs: Any): ...

_T = TypeVar("_T")

class cached_property(Generic[_T]):
    func: Callable[..., _T] = ...
    __doc__: Any = ...
    name: str = ...
    def __init__(self, func: Callable[..., _T], name: Optional[str] = ...): ...
    @overload
    def __get__(self, instance: None, cls: Type[Any] = ...) -> "cached_property[_T]": ...
    @overload
    def __get__(self, instance: object, cls: Type[Any] = ...) -> _T: ...

class Promise: ...

def lazy(func: Union[Callable, Type[str]], *resultclasses: Any) -> Callable: ...
def lazystr(text: Any): ...
def keep_lazy(*resultclasses: Any) -> Callable: ...
def keep_lazy_text(func: Callable) -> Callable: ...

empty: Any

def new_method_proxy(func: Any): ...

class LazyObject:
    def __init__(self) -> None: ...
    __getattr__: Any = ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...
    def __reduce__(self) -> Tuple[Callable, Tuple[Model]]: ...
    def __copy__(self): ...
    def __deepcopy__(self, memo: Any): ...
    __bytes__: Any = ...
    __bool__: Any = ...
    __dir__: Any = ...
    __class__: Any = ...
    __ne__: Any = ...
    __hash__: Any = ...
    __getitem__: Any = ...
    __setitem__: Any = ...
    __delitem__: Any = ...
    __iter__: Any = ...
    __len__: Any = ...
    __contains__: Any = ...

def unpickle_lazyobject(wrapped: Model) -> Model: ...

class SimpleLazyObject(LazyObject):
    def __init__(self, func: Callable) -> None: ...
    def __copy__(self) -> List[int]: ...
    def __deepcopy__(self, memo: Dict[Any, Any]) -> List[int]: ...

_PartitionMember = TypeVar("_PartitionMember")

def partition(
    predicate: Callable, values: List[_PartitionMember]
) -> Tuple[List[_PartitionMember], List[_PartitionMember]]: ...

_Get = TypeVar("_Get")
_Self = TypeVar("_Self")

class classproperty(Generic[_Get]):
    fget: Optional[Callable[[Type[_Self]], _Get]] = ...
    def __init__(self, method: Optional[Callable[[Type[_Self]], _Get]] = ...) -> None: ...
    @overload
    def __get__(self, instance: None, cls: Type[_Self] = ...) -> _Get: ...
    @overload
    def __get__(self, instance: _Self, cls: Type[_Self] = ...) -> _Get: ...
    def getter(self, method: Callable[[Type[_Self]], _Get]) -> classproperty[_Get]: ...



import sys
from collections import deque
from types import TracebackType
from typing import Any, Awaitable, Callable, Generator, Type, TypeVar

from .events import AbstractEventLoop
from .futures import Future

_T = TypeVar("_T")

if sys.version_info >= (3, 9):
    class _ContextManagerMixin:
        def __init__(self, lock: Lock | Semaphore) -> None: ...
        def __aenter__(self) -> Awaitable[None]: ...
        def __aexit__(
            self, exc_type: Type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None
        ) -> Awaitable[None]: ...

else:
    class _ContextManager:
        def __init__(self, lock: Lock | Semaphore) -> None: ...
        def __enter__(self) -> object: ...
        def __exit__(self, *args: Any) -> None: ...
    class _ContextManagerMixin:
        def __init__(self, lock: Lock | Semaphore) -> None: ...
        # Apparently this exists to *prohibit* use as a context manager.
        def __enter__(self) -> object: ...
        def __exit__(self, *args: Any) -> None: ...
        def __iter__(self) -> Generator[Any, None, _ContextManager]: ...
        def __await__(self) -> Generator[Any, None, _ContextManager]: ...
        def __aenter__(self) -> Awaitable[None]: ...
        def __aexit__(
            self, exc_type: Type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None
        ) -> Awaitable[None]: ...

class Lock(_ContextManagerMixin):
    def __init__(self, *, loop: AbstractEventLoop | None = ...) -> None: ...
    def locked(self) -> bool: ...
    async def acquire(self) -> bool: ...
    def release(self) -> None: ...

class Event:
    def __init__(self, *, loop: AbstractEventLoop | None = ...) -> None: ...
    def is_set(self) -> bool: ...
    def set(self) -> None: ...
    def clear(self) -> None: ...
    async def wait(self) -> bool: ...

class Condition(_ContextManagerMixin):
    def __init__(self, lock: Lock | None = ..., *, loop: AbstractEventLoop | None = ...) -> None: ...
    def locked(self) -> bool: ...
    async def acquire(self) -> bool: ...
    def release(self) -> None: ...
    async def wait(self) -> bool: ...
    async def wait_for(self, predicate: Callable[[], _T]) -> _T: ...
    def notify(self, n: int = ...) -> None: ...
    def notify_all(self) -> None: ...

class Semaphore(_ContextManagerMixin):
    _value: int
    _waiters: deque[Future[Any]]
    def __init__(self, value: int = ..., *, loop: AbstractEventLoop | None = ...) -> None: ...
    def locked(self) -> bool: ...
    async def acquire(self) -> bool: ...
    def release(self) -> None: ...
    def _wake_up_next(self) -> None: ...

class BoundedSemaphore(Semaphore):
    def __init__(self, value: int = ..., *, loop: AbstractEventLoop | None = ...) -> None: ...



from typing import Any

from django.conf import settings as settings
from django.core.management.base import BaseCommand as BaseCommand
from django.core.management.utils import get_command_line_option as get_command_line_option
from django.test.utils import get_runner as get_runner

class Command(BaseCommand):
    test_runner: Any = ...
    def run_from_argv(self, argv: Any) -> None: ...
    def add_arguments(self, parser: Any) -> None: ...



from typing import Any

from django.db.backends.base.features import BaseDatabaseFeatures as BaseDatabaseFeatures

class DatabaseFeatures(BaseDatabaseFeatures):
    interprets_empty_strings_as_nulls: bool = ...
    has_select_for_update: bool = ...
    has_select_for_update_nowait: bool = ...
    has_select_for_update_skip_locked: bool = ...
    has_select_for_update_of: bool = ...
    select_for_update_of_column: bool = ...
    can_return_columns_from_insert: bool = ...
    can_introspect_autofield: bool = ...
    supports_subqueries_in_group_by: bool = ...
    supports_transactions: bool = ...
    supports_timezones: bool = ...
    has_native_duration_field: bool = ...
    can_defer_constraint_checks: bool = ...
    supports_partially_nullable_unique_constraints: bool = ...
    supports_deferrable_unique_constraints: bool = ...
    truncates_names: bool = ...
    supports_tablespaces: bool = ...
    supports_sequence_reset: bool = ...
    can_introspect_materialized_views: bool = ...
    can_introspect_time_field: bool = ...
    atomic_transactions: bool = ...
    supports_combined_alters: bool = ...
    nulls_order_largest: bool = ...
    requires_literal_defaults: bool = ...
    closed_cursor_error_class: Any = ...
    bare_select_suffix: str = ...
    supports_select_for_update_with_limit: bool = ...
    supports_temporal_subtraction: bool = ...
    ignores_table_name_case: bool = ...
    supports_index_on_text_field: bool = ...
    has_case_insensitive_like: bool = ...
    create_test_procedure_without_params_sql: str = ...
    create_test_procedure_with_int_param_sql: str = ...
    supports_callproc_kwargs: bool = ...
    supports_over_clause: bool = ...
    supports_frame_range_fixed_distance: bool = ...
    supports_ignore_conflicts: bool = ...
    max_query_params: Any = ...
    supports_partial_indexes: bool = ...
    supports_slicing_ordering_in_compound: bool = ...
    allows_multiple_constraints_on_same_fields: bool = ...
    supports_boolean_expr_in_select_clause: bool = ...
    supports_primitives_in_json_field: bool = ...
    supports_json_field_contains: bool = ...



from ctypes import Structure
from typing import Any, Optional

logger: Any

def load_geos(): ...

NOTICEFUNC: Any

def notice_h(fmt: Any, lst: Any) -> None: ...

ERRORFUNC: Any

def error_h(fmt: Any, lst: Any) -> None: ...

class GEOSGeom_t(Structure): ...
class GEOSPrepGeom_t(Structure): ...
class GEOSCoordSeq_t(Structure): ...
class GEOSContextHandle_t(Structure): ...

GEOM_PTR: Any
PREPGEOM_PTR: Any
CS_PTR: Any
CONTEXT_PTR: Any
lgeos: Any

class GEOSFuncFactory:
    argtypes: Any = ...
    restype: Any = ...
    errcheck: Any = ...
    func_name: Any = ...
    def __init__(
        self,
        func_name: Any,
        *,
        restype: Optional[Any] = ...,
        errcheck: Optional[Any] = ...,
        argtypes: Optional[Any] = ...
    ) -> None: ...
    def __call__(self, *args: Any): ...
    def func(self): ...

def geos_version(): ...
def geos_version_tuple(): ...



from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Union

from django.db.models.sql.where import NothingNode

_NodeChildren = Iterable[Union["Node", NothingNode, Sequence[Any]]]

class Node:
    children: List[Any]
    default: Any = ...
    connector: str = ...
    negated: bool = ...
    def __init__(
        self, children: Optional[_NodeChildren] = ..., connector: Optional[str] = ..., negated: bool = ...
    ) -> None: ...
    def __deepcopy__(self, memodict: Dict[Any, Any]) -> Node: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __contains__(self, other: Tuple[str, int]) -> bool: ...
    def __hash__(self) -> int: ...
    def add(self, data: Any, conn_type: str, squash: bool = ...) -> Any: ...
    def negate(self) -> None: ...



from io import IOBase
from typing import Any, Dict, Optional, Tuple

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.utils import CursorDebugWrapper as BaseCursorDebugWrapper

def psycopg2_version() -> Tuple[int, ...]: ...

PSYCOPG2_VERSION: Tuple[int, ...] = ...

class DatabaseWrapper(BaseDatabaseWrapper):
    operators: Dict[str, str] = ...
    pattern_esc: str = ...
    pattern_ops: Dict[str, str] = ...

    # PostgreSQL backend-specific attributes.
    _named_cursor_idx: int = ...
    @property
    def pg_version(self) -> str: ...

class CursorDebugWrapper(BaseCursorDebugWrapper):
    def copy_expert(self, sql: str, file: IOBase, *args: Any): ...
    def copy_to(self, file: IOBase, table: str, *args: Any, **kwargs: Any): ...



import sys
from datetime import datetime
from enum import Enum
from typing import IO, Any, Mapping, MutableMapping, Type

class PlistFormat(Enum):
    FMT_XML: int
    FMT_BINARY: int

FMT_XML = PlistFormat.FMT_XML
FMT_BINARY = PlistFormat.FMT_BINARY

if sys.version_info >= (3, 9):
    def load(fp: IO[bytes], *, fmt: PlistFormat | None = ..., dict_type: Type[MutableMapping[str, Any]] = ...) -> Any: ...
    def loads(value: bytes, *, fmt: PlistFormat | None = ..., dict_type: Type[MutableMapping[str, Any]] = ...) -> Any: ...

else:
    def load(
        fp: IO[bytes],
        *,
        fmt: PlistFormat | None = ...,
        use_builtin_types: bool = ...,
        dict_type: Type[MutableMapping[str, Any]] = ...,
    ) -> Any: ...
    def loads(
        value: bytes,
        *,
        fmt: PlistFormat | None = ...,
        use_builtin_types: bool = ...,
        dict_type: Type[MutableMapping[str, Any]] = ...,
    ) -> Any: ...

def dump(
    value: Mapping[str, Any] | list[Any] | tuple[Any, ...] | str | bool | float | bytes | datetime,
    fp: IO[bytes],
    *,
    fmt: PlistFormat = ...,
    sort_keys: bool = ...,
    skipkeys: bool = ...,
) -> None: ...
def dumps(
    value: Mapping[str, Any] | list[Any] | tuple[Any, ...] | str | bool | float | bytes | datetime,
    *,
    fmt: PlistFormat = ...,
    skipkeys: bool = ...,
    sort_keys: bool = ...,
) -> bytes: ...

if sys.version_info < (3, 9):
    def readPlist(pathOrFile: str | IO[bytes]) -> Any: ...
    def writePlist(value: Mapping[str, Any], pathOrFile: str | IO[bytes]) -> None: ...
    def readPlistFromBytes(data: bytes) -> Any: ...
    def writePlistToBytes(value: Mapping[str, Any]) -> bytes: ...

if sys.version_info < (3, 7):
    class Dict(dict[str, Any]):
        def __getattr__(self, attr: str) -> Any: ...
        def __setattr__(self, attr: str, value: Any) -> None: ...
        def __delattr__(self, attr: str) -> None: ...

if sys.version_info < (3, 9):
    class Data:
        data: bytes
        def __init__(self, data: bytes) -> None: ...

if sys.version_info >= (3, 8):
    class UID:
        data: int
        def __init__(self, data: int) -> None: ...
        def __index__(self) -> int: ...
        def __reduce__(self) -> Any: ...
        def __hash__(self) -> int: ...

class InvalidFileException(ValueError):
    def __init__(self, message: str = ...) -> None: ...



from typing import Any

from django.core.handlers import base as base
from django.http import HttpRequest as HttpRequest

logger: Any

class ASGIRequest(HttpRequest):
    body_receive_timeout: int = ...
    scope: Any = ...
    resolver_match: Any = ...
    script_name: Any = ...
    path_info: Any = ...
    path: Any = ...
    method: Any = ...
    META: Any = ...
    def __init__(self, scope: Any, body_file: Any) -> None: ...
    def GET(self): ...
    POST: Any = ...
    FILES: Any = ...
    def COOKIES(self): ...

class ASGIHandler(base.BaseHandler):
    request_class: Any = ...
    chunk_size: Any = ...
    def __init__(self) -> None: ...
    async def __call__(self, scope: Any, receive: Any, send: Any) -> None: ...
    async def read_body(self, receive: Any): ...
    def create_request(self, scope: Any, body_file: Any): ...
    def handle_uncaught_exception(self, request: Any, resolver: Any, exc_info: Any): ...
    async def send_response(self, response: Any, send: Any) -> None: ...
    @classmethod
    def chunk_bytes(cls, data: Any) -> None: ...
    def get_script_prefix(self, scope: Any): ...
